Attribute VB_Name = "Module1"
Option Explicit


' =====================================================================
' privates
' =====================================================================

' constants
' =====================================================================
Private Const S_OK          As Long = 0
Private Const BI_RGB        As Long = 0
Private Const IPictureIID   As String = "{7BF80980-BF32-101A-8BBB-00AA00300CAB}"

Private Type BITMAPFILEHEADER
    bfType                      As Integer
    bfSize                      As Long
    bfReserved1                 As Integer
    bfReserved2                 As Integer
    bfOffBits                   As Long
End Type



' =====================================================================
' privates
' =====================================================================
Public Type BITMAPINFOHEADER
    biSize                      As Long
    biWidth                     As Long
    biHeight                    As Long
    biPlanes                    As Integer
    biBitCount                  As Integer
    biCompression               As Long
    biSizeImage                 As Long
    biXPelsPerMeter             As Long
    biYPelsPerMeter             As Long
    biClrUsed                   As Long
    biClrImportant              As Long
End Type



' =====================================================================
' externals
' =====================================================================
Private Declare Function OLE_CID2BAR _
    Lib "ole32" Alias "CLSIDFromString" _
   (ByVal lpszProgID As Long, _
    ByRef pCLSID As Any) As Long

Private Declare Function OLE_CreateStream _
    Lib "ole32" Alias "CreateStreamOnHGlobal" _
   (ByRef hGlobal As Any, _
    ByVal fDeleteOnRelease As Boolean, _
    ByRef ppstm As Any) As Long

Private Declare Function OLE_LoadPicture _
    Lib "oleaut32" Alias "OleLoadPicture" _
   (ByVal lpstream As Long, _
    ByVal lSize As Long, _
    ByVal fRunmode As Long, _
    ByRef riid As Any, _
    ByRef lplpvObj As Any) As Long
         
Private Declare Sub memCPY _
    Lib "kernel32" Alias "RtlMoveMemory" _
   (ByRef hpvDest As Any, _
    ByRef hpvSource As Any, _
    ByVal cbCopy As Long)

' TIP654        Bild aus im Speicher liegenden Daten erstellen
'
' PTR2STD       create picture object from any data
'               (softKUS) - I/2005
'
' CALL:         PTR2STD(wid, hgt, bpp, ptr, @std, [@ecd])
'
' IN:           lng:wid width
'               lng:hgt height
'               lng:bpp bits/pixel
'               lng:ptr points to bitmap data
'               obj:std picture object to be created
'               lng:ecd error code
'                       -1: ole error
'                       -2: stream object creation error
'                       -3: picture creation failed
'
' OUT:          bol     success
'
Function PTR2STD( _
    wid As Long, _
    hgt As Long, _
    bpp As Long, _
    ptr As Long, _
    std As StdPicture, _
    Optional ecd As Long) As Boolean
    
    Dim bfh     As BITMAPFILEHEADER
    Dim bmi     As BITMAPINFOHEADER
    Dim pic     As IPicture     ' picture object
    Dim obj     As IUnknown     ' stream object
    Dim buf()   As Byte         ' stream buffer
    Dim iid(15) As Byte         ' IPicture-guid
    
    ' set BitmapInfoHeader according to parameters
    With bmi
        .biSize = Len(bmi)
        .biCompression = BI_RGB
        .biBitCount = bpp
        .biWidth = wid
        .biHeight = hgt
        .biPlanes = 1
        .biSizeImage = ((wid * bpp / 8 + 3) And -4&) * hgt
    End With
    
    ' ... and the BitmapFileHeader
    With bfh
        .bfType = &H4D42    ' "BM"
        .bfOffBits = Len(bfh) + Len(bmi)
        .bfSize = .bfOffBits + bmi.biSizeImage
    End With
    
    ' Create bitmap file information within memory
    ' 1st: allocate memory
    ReDim buf(bfh.bfSize - 1)
    
    ' 2nd: copy BitmapFileHeader (take care of pad bytes!)
    memCPY buf(0), bfh, 2
    memCPY buf(2), ByVal VarPtr(bfh.bfSize), Len(bfh) - 2
            
    ' 3rd: copy BitmapInfoHeader
    memCPY buf(Len(bfh)), bmi, Len(bmi)
    
    ' 4th: copy data
    memCPY buf(Len(bfh) + Len(bmi)), ByVal ptr, bmi.biSizeImage

    ' convert iid
    If OLE_CID2BAR(StrPtr(IPictureIID), iid(0)) <> S_OK Then
        ecd = -1        ' ole error
    
    ' create stream
    ElseIf OLE_CreateStream(buf(0), False, obj) <> S_OK Then
        ecd = -2        ' stream object creation error

    ' create picture
    ElseIf OLE_LoadPicture(ObjPtr(obj), bfh.bfSize, False, iid(0), _
                           pic) <> S_OK Then
        ecd = -3        ' picture creation failed
    
    Else
        ecd = 0         ' error code 0
        Set std = pic   ' set target
        PTR2STD = True   ' return success
    End If
    
    Set pic = Nothing
    Set obj = Nothing
End Function

' STD2BAR   copy a standard picture's data to a byte array
'           (softKUS) - IV/2005
'
' CALL:     STD2BAR(std, @bmi, @bar, [adj], [@ecd])
'
' IN:       obj:std standard picture object
'           bol:adj .T.: remove bmf/bmi from array data
'                   .F.: do not change array data (dflt)
'
' OUT:      arr:bar target byte array
'           udt:bmi bitmapinfoheader filled acc.
'           lng:ecd error code
'                   -3: picture creation failed
'                   -4: saving object to stream failed
'
' RET:      bol     success
'
Function STD2BAR( _
    std As IPicture, _
    bmi As BITMAPINFOHEADER, _
    bar() As Byte, _
    Optional adj As Boolean, _
    Optional ecd As Long) As Boolean
    
    Dim stm     As IUnknown
    Dim bmf     As BITMAPFILEHEADER
    Dim lmx     As Long
    Dim wid     As Long
    Dim hgt     As Long
    Dim tmp     As Long
    Dim hmX     As Single
    Dim hmY     As Single
    
    ecd = 0
    
    ' calculate space needed
    ' assume maximal size  = 4 BytesPerPixel
    ' --------------------------------------------------------------------------
    hmX = 0.567 / Screen.TwipsPerPixelX ' HighMetric to vbPixel - X
    hmY = 0.567 / Screen.TwipsPerPixelY ' HighMetric to vbpixel - Y
    
    wid = (std.Width * hmX * 4 + 3) And -4&
    hgt = std.Height * hmY
    ReDim bar(Len(bmi) + Len(bmf) + wid * hgt)
    
    If OLE_CreateStream(bar(0), False, stm) <> S_OK Then
        ecd = -3    ' stream object creation error
            
    Else
        std.SaveAsFile ByVal ObjPtr(stm), True, lmx
        
        If lmx = 0 Then
            ecd = -4 ' saving object to stream failed
    
        Else
            memCPY bmi, bar(Len(bmf)), Len(bmi)
            
            If adj Then
                tmp = Len(bmf) + Len(bmi)
                memCPY bar(0), bar(tmp), lmx - tmp
            End If
            
            ReDim Preserve bar(lmx - tmp - 1)
            STD2BAR = True
        End If
    End If
    
    Set stm = Nothing
    Set std = Nothing
End Function

